﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Operators;
using HeuristicLab.Common;
using HeuristicLab.Parameters;

namespace HeuristicLab.Algorithms.CellularGeneticAlgorithm {
  [Item("NeighborhoodSelector", "Selects a neighborhood for the cellular GA.")]
  [StorableClass]
  public abstract class NeighborhoodSelector: SingleSuccessorOperator {
    protected ScopeParameter CurrentScopeParameter {
      get { return (ScopeParameter)Parameters["CurrentScope"]; }
    }

    public IScope CurrentScope {
      get { return CurrentScopeParameter.ActualValue; }
    }

    [StorableConstructor]
    protected NeighborhoodSelector(bool deserializing) : base(deserializing) { }

    public NeighborhoodSelector()
      : base() {
        Parameters.Add(new ScopeParameter("CurrentScope", "The current scope for which sub-scopes should be selected."));
    }

    protected NeighborhoodSelector(NeighborhoodSelector original, Cloner cloner)
      : base(original, cloner) {
    }

    protected IScope ShallowCopy(IScope scope) {
      IScope copy = new Scope(scope.Name, scope.Description);
      foreach (IVariable variable in scope.Variables) {
        Variable copyVar = new Variable();
        copyVar.Name = variable.Name;
        copyVar.Value = variable.Value;
        copy.Variables.Add(copyVar);
      }

      return copy;
    }

    protected abstract List<IScope> Select(IScope individual, List<IScope> population);

    public override IOperation Apply() {
      IScope individual = CurrentScope;
      List<IScope> population = new List<IScope>(CurrentScope.Parent.SubScopes);

      IScope root = new Scope();
      CurrentScope.SubScopes.Add(root);

      IScope root2 = ShallowCopy(individual);
      root.SubScopes.Add(ShallowCopy(individual));
      root.SubScopes.Add(root2);

      IScope root3 = new Scope();
      IScope root4 = new Scope();
      root2.SubScopes.Add(root3);
      root2.SubScopes.Add(root4);

      root3.SubScopes.Add(ShallowCopy(individual));
      foreach (IScope neighbor in Select(individual, population)) {
        root4.SubScopes.Add(ShallowCopy(neighbor));
      }
      
      return base.Apply();
    }
  }
}
